import unittest
from cy_ioc_extract import IOCEXtract
from cy_ioc_extract.exception import UnsupportedExtractFieldError


class TestIOCExtract(unittest.TestCase):
    def assertListEqualBySort(self, list1, list2):
        list1.sort()
        list2.sort()
        self.assertListEqual(list1, list2)

    @staticmethod
    def compare_dicts(dict1, dict2):
        """
        Compares two dictionaries where values are lists of strings.
        The comparison is case-insensitive and ignores the order of values.

        Returns True if both dictionaries are equal, otherwise False.
        """
        if dict1.keys() != dict2.keys():
            return False  # Keys must be the same

        for key in dict1:
            values1 = set(val.lower() for val in dict1[key])  # Normalize case
            values2 = set(val.lower() for val in dict2[key])  # Normalize case

            if values1 != values2:
                return False  # Values must be the same
        return True  # If everything matches

    def setUp(self):
        self.sample_text = """
        ### IPv4 Addresses:
        192.168.1.1
        10.0.0.1
        8.8.8.8
        172.16.32.45

        ### Domains:
        example.com
        subdomain.test.org
        my-site.net
        web.co.uk

        ### Email Addresses:
        john.doe@example.com
        alice_smith@corporate.org
        user123@test.net
        contact@web.co.uk
        """

    def test_extract_specific_fields(self):
        extractor = IOCEXtract(self.sample_text, extract_fields=("DOMAIN", "EMAIL"))
        result = extractor.extract_ioc()
        expected = {
            "DOMAIN": ["example.com", "subdomain.test.org", "my-site.net", "web.co.uk"],
            "EMAIL": [
                "john.doe@example.com",
                "alice_smith@corporate.org",
                "user123@test.net",
                "contact@web.co.uk"
            ]
        }
        assert self.compare_dicts(expected, result) is True

    def test_extract_all_fields(self):
        extractor = IOCEXtract(self.sample_text)
        result = extractor.extract_ioc()
        self.assertIn("IP", result)
        self.assertIn("DOMAIN", result)
        self.assertIn("EMAIL", result)

    def test_extract_without_validation(self):
        extractor = IOCEXtract(self.sample_text, extract_fields=("DOMAIN",), validate_ioc=False)
        result = extractor.extract_ioc()
        expected = {"DOMAIN": ["192.168", "172.16.32.45", "example.com", "subdomain.test.org", "my-site.net", "web.co.uk"]}
        assert self.compare_dicts(result, expected) is True

    def test_extract_invalid_field(self):
        with self.assertRaises(UnsupportedExtractFieldError):
            IOCEXtract(self.sample_text, extract_fields=("DOMAIN", "INVALID_FIELD")).extract_ioc()

    def test_extract_ip_addresses(self):
        extractor = IOCEXtract(self.sample_text, extract_fields=("IP",))
        result = extractor.extract_ioc()
        expected = {"IP": ["192.168.1.1", "10.0.0.1", "8.8.8.8", "172.16.32.45"]}
        assert self.compare_dicts(result, expected) is True

    def test_extract_empty_text(self):
        extractor = IOCEXtract("")
        result = extractor.extract_ioc()
        for key, val in result.items():
            self.assertEqual(len(val), 0)

    def test_extract_urls(self):
        sample_text = "Check this site: https://example.com and ftp://192.168.1.1/resource"
        extractor = IOCEXtract(sample_text, extract_fields=("URL",))
        result = extractor.extract_ioc()
        expected = {"URL": ["https://example.com", "ftp://192.168.1.1/resource"]}
        assert self.compare_dicts(result, expected) is True

    def test_extract_hashes(self):
        sample_text = "SHA256: d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2"
        extractor = IOCEXtract(sample_text, extract_fields=("SHA256",))
        result = extractor.extract_ioc()
        expected = {"SHA256": ["d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2"]}
        assert self.compare_dicts(result, expected) is True

    def test_csap_use_cases(self):
        test_cases = {
            1: ("URL", "https://google.def.com icq.im/XY8ejJTQiWg8", {"URL": ["https://google.def.com", "icq.im/XY8ejJTQiWg8"]}),
            2: ("DOMAIN", "google.COM, something.LIBRE, something.DYN", {"DOMAIN": ["google.COM", "something.LIBRE", "something.DYN"]}),
            3: ("EMAIL", "john.smith(comment)@example.com deepak.kumar+1@cyware.com", {"EMAIL": ["john.smith(comment)@example.com", "deepak.kumar+1@cyware.com"]}),
        }
        for _, test_case in test_cases.items():
            extractor = IOCEXtract(test_case[1], extract_fields=(test_case[0],))
            result = extractor.extract_ioc()
            assert self.compare_dicts(result, test_case[2]) is True


if __name__ == "__main__":
    unittest.main()
